
//=============================================================================

#include <math.h>

#include "airport.h"

//=============================================================================

int airport_get_ground(void)
 {
  return 8; // Default ground frequency
 }

int airport_get_tower(void)
 {
  return 5; // Default tower frequency
 }

int airport_get_startlot(void)
 {
  // This is airport specific.
  // Returns the "vertex" corresponding to where planes should start out
  // on the airport.
  return 12;
 }

int airport_get_endlot(void)
 {
  // This is airport specific.
  // Returns the "vertex" corresponding to where planes should
  // park (after landing).
  return 34;
 }

//=============================================================================

// The table below lists each point's (X,Y) values, and their neighbors.
// For example:
//     The 1st entry corresponds to "Point 0".
//     And it's at (298,421)
//     And its neighbors are Point5, Point9, and Point20.
//     (The "-1" value signifies the end of the "Neighbor" list)
airport_point airport_points[airport_max] =
 {{298,421,{ 5,19,20   ,-1}} //  0
 ,{391,277,{ 8         ,-1}} //  1
 ,{429,216,{ 9         ,-1}} //  2
 ,{446,186,{10,33      ,-1}} //  3
 ,{242,454,{ 5,15,23,27,-1}} //  4
 ,{269,414,{ 0, 4, 6   ,-1}} //  5
 ,{295,376,{ 5, 7,26   ,-1}} //  6
 ,{318,340,{ 6, 8, 25  ,-1}} //  7
 ,{367,262,{ 1, 7, 9   ,-1}} //  8
 ,{405,200,{ 2, 8,10   ,-1}} //  9
 ,{422,172,{ 3, 9,11   ,-1}} // 10
 ,{437,143,{10,24      ,-1}} // 11
 ,{381,483,{13         ,-1}} // 12
 ,{359,469,{12,20      ,-1}} // 13
 ,{145,640,{15,23      ,-1}} // 14
 ,{129,631,{ 4,14,21   ,-1}} // 15
 ,{ 52,585,{17,21      ,-1}} // 16
 ,{ 92,410,{16,18,28   ,-1}} // 17
 ,{ 91,390,{17,22      ,-1}} // 18
 ,{401,609,{ 0,23      ,-1}} // 19
 ,{337,432,{ 0,13      ,-1}} // 20
 ,{ 63,595,{15,16,28,30,-1}} // 21
 ,{135,248,{18,25,27,31,-1}} // 22
 ,{339,640,{ 4,14,19,29,-1}} // 23
 ,{385,107,{11,25,32   ,-1}} // 24
 ,{278,268,{ 7,22,24,26,-1}} // 25
 ,{254,303,{ 6,25,27   ,-1}} // 26
 ,{202,380,{ 4,22,26,28,-1}} // 27
 ,{183,410,{17,21,27   ,-1}} // 28
 ,{371,697,{23         ,-1}} // 29
 ,{ 10,671,{21         ,-1}} // 30
 ,{ 32, 37,{22         ,-1}} // 31
 ,{439, 19,{24         ,-1}} // 32
 ,{494,218,{ 3,34      ,-1}} // 33
 ,{412,350,{33         ,-1}} // 34
 };

// Each of the 36 entries correspond to a list { a, b, c, d,... ,-1}
// such that these are the (consecutive) points on the runway 0..35
// (where "a" is the starting point...}
//
// If a runway does not exist, its list is { -1 }
static int local_runway[36][10]
 = { {                        -1} //  0
   , {                        -1} //  1
   , {                        -1} //  2
   , {                        -1} //  3
   , {                        -1} //  4
   , {29,23,4,27,22,31,       -1} //  5
   , {                        -1} //  6
   , {                        -1} //  7
   , {                        -1} //  8
   , {                        -1} //  9
   , {                        -1} // 10
   , {30,21,28,27,26,25,24,32,-1} // 11
   , {                        -1} // 12
   , {                        -1} // 13
   , {                        -1} // 14
   , {                        -1} // 15
   , {                        -1} // 16
   , {                        -1} // 17
   , {                        -1} // 18
   , {                        -1} // 19
   , {                        -1} // 20
   , {                        -1} // 21
   , {                        -1} // 22
   , {31,22,27,4,23,29,       -1} // 23
   , {                        -1} // 24
   , {                        -1} // 25
   , {                        -1} // 26
   , {                        -1} // 27
   , {                        -1} // 28
   , {32,24,25,26,27,28,21,30,-1} // 29
   , {                        -1} // 30
   , {                        -1} // 31
   , {                        -1} // 32
   , {                        -1} // 33
   , {                        -1} // 34
   , {                        -1} // 35
   };

// Each of the 26 entries correspond to a list { a, b, c, d,... ,-1}
// such that these are the (consecutive) points on the taxiway A..Z.
//
// If a taxiway does not exist, its list is { -1 }
static int local_taxiway[26][10]
 = { {1,8,               -1} // Alpha
   , {2,9,               -1} // Bravo
   , {3,10,              -1} // Charlie
   , {                   -1} // Delta
   , {4,5,6,7,8,9,10,11, -1} // Echo
   , {6,26,              -1} // Foxtrot
   , {7,25,              -1} // Golf
   , {22,25,             -1} // Hotel
   , {                   -1} // India
   , {20,13,12,          -1} // Juliet
   , {                   -1} // Kilo
   , {                   -1} // Lima
   , {21,15,14,23,       -1} // Mike
   , {17,28,             -1} // November
   , {                   -1} // Oscar
   , {                   -1} // Papa
   , {                   -1} // Quebec
   , {22,18,17,16,21,    -1} // Romeo
   , {0,19,23,           -1} // Sierra
   , {5,0,20,            -1} // Tango
   , {                   -1} // Uniform
   , {                   -1} // Victor
   , {4,15,              -1} // Whisky
   , {                   -1} // X-Ray
   , {                   -1} // Yankee
   , {                   -1} // Zulu
   };

//=============================================================================

// Given runway 0..35, it returns its starting point.
// If no such runway, it returns (-1);
int airport_runway_to_point(int runway)
 {
  if (runway<0 || runway>35) return (-1);
  if (local_runway[runway][0]<0) return (-1); return local_runway[runway][1];
 }

// Given runway 0..35, it returns a list of its points
// If no such runway, it returns NULL
int* airport_runway_to_points(int runway)
 {
  int* answer;
  if (runway>=0 && runway<=35)
   {
    answer=local_runway[runway];
    if ((*answer)>=0) return answer;
   }
  return 0;
 }

// Given taxiway 1..26, it returns a list of its points
// If no such taxiway, it returns NULL
int* airport_taxiway_to_points(int taxiway)
 {
  int* answer;
  if (taxiway>=1 && taxiway<=26)
   {
    answer=local_taxiway[taxiway-1];
    if ((*answer)>=0) return answer;
   }
  return 0;
 }

//=============================================================================

// NonZero if and only if "airport_dist" is already computed.
static int local_costcomputed=0;

// The raw distance between each pair of points
int airport_dist[airport_max][airport_max];

//=============================================================================

// Given SRC and DEST,
// this uses Dijkstra's Shortest Path algorithm to find the shortest path.
//
// If reachable: It writes { SRC , ... , DEST , -1 } into result[0..]
//               And return the number of points (not counting "-1")
//
// If not reachable: result[0..] are unmodified. It returns 0.
//
int airport_optimize(int src,int dest,int* result)
 {
  int maxcost,i,j,len;
  int cost[airport_max],set[airport_max],local_pred[airport_max],setn;
  // Step 0A: Compute the maximum positive cost
  maxcost=1;
  while(1) { int x=(maxcost+maxcost); if (x<0) break; maxcost=x; }
  maxcost=maxcost+(maxcost-1);
  // Step 0B: Compute the distance matrix (if they're not already computed).
  if (local_costcomputed==0)
   {
    local_costcomputed=1;
    // First compute the raw distance between points
    for(i=0;i<airport_max;i++)
     {
      for(j=0;j<airport_max;j++)
       {
        int dx=(airport_points[i].x - airport_points[j].x);
        int dy=(airport_points[i].y - airport_points[j].y);
        if (i==j) dx=0; else dx=((int)(sqrt((double)(dx*dx+dy*dy))));
        if (dx<0) dx=0;
        airport_dist[i][j]=dx;
       }
     }
    // Here, we want to dramatically increase the cost of going on the runways
    for(i=0;i<36;i++)
     {
      j=0;
      if (local_runway[i][0]<0) continue;
      while(local_runway[i][j+1]>=0)
       {
        int a=local_runway[i][j]; int b=local_runway[i][j+1];
        airport_dist[a][b]=10*1500*1500; airport_dist[a][b]=10*1500*1500;
        j++;
       }
     }
   }
  // Step 1: Initialize the "costs", "set", and "predecessor" array.
  for(i=0;i<airport_max;i++) { set[i]=i; local_pred[i]=(-1); cost[i]=maxcost; }
  cost[src]=0; setn=airport_max;
  // Step 2: While set Q is not empty...
  while(setn>0)
   {
    int minchoice=0; int minvalue=cost[set[0]];
    // Pick the cheapest element "minchoice"...
    for(i=1;i<setn;i++)
     {
      int x=set[i]; if (minvalue>cost[x]) { minvalue=cost[x]; minchoice=i; }
     }
    // Remove the chosen element from the set
    minvalue=set[minchoice]; set[minchoice]=set[setn-1]; setn=setn-1;
    minchoice=minvalue;
    // For each of "minchoice"'s neighbor...
    for(i=0;i<airport_max;i++)
     {
      int x=airport_points[minchoice].neighbor[i]; if (x<0) break;
      minvalue=cost[minchoice]+airport_dist[minchoice][x];
      if (minvalue<0) continue; // That means the cost is TOO HIGH
      if (cost[x]>minvalue) { cost[x]=minvalue; local_pred[x]=minchoice; }
     }
   }
  // Step 3: Count the # of points on this PATH...
  len=1; j=dest;
  while(j!=src)
   {
    len=len+1; j=local_pred[j]; if (j<0) return -1; // DEST is unreachable!
   }
  // Step 4: Return the path...
  result[len]=(-1); i=len-1; j=dest;
  while(1)
   {
    result[i]=j;
    if (j==src) return len; // This means DEST is reachable
    i--; j=local_pred[j];
   }
 }

//=============================================================================
