
#ifndef include_airport_h
#define include_airport_h

//=============================================================================

// The maximum number of points on the airport
#define airport_max 35

//=============================================================================

// This structure represents 1 vertex (and its neighbors)
typedef struct
 {
  int x,y; int neighbor[10];
 }
airport_point;

// This array stores each vertex
extern airport_point airport_points[];

//=============================================================================

// This array contains the cost between 2 points
//
// (Note: This array will only become valid after the first call to
// "airport_optimize()". After a call to "airport_optimize", this array
// will stay valid and usable for the rest of the time.)
//
extern int airport_dist[airport_max][airport_max];

//=============================================================================

// Given SRC and DEST,
// this uses Dijkstra's Shortest Path algorithm to find the shortest path.
//
// If reachable: It writes { SRC , ... , DEST , -1 } into result[0..]
//               And return the number of points (not counting "-1")
//
// If not reachable: result[0..] are unmodified. It returns -1.
//
// Note: result must be able to store at least "airport_max+1" integers.
//
int airport_optimize(int src,int dest,int* result);

//=============================================================================

// Returns the ground frequency
int airport_get_ground(void);

// Returns the tower frequency
int airport_get_tower(void);

// Returns the "vertex" corresponding to where planes should start out
// on the airport.
int airport_get_startlot(void);

// Returns the "vertex" corresponding to where planes should
// park (after landing).
int airport_get_endlot(void);

//=============================================================================

// Given runway 0..35, it returns its starting point.
// If no such runway, it returns (-1);
//
int airport_runway_to_point(int runway);

// Given runway 0..35, it returns a list { a, b, c, d,... ,-1}
// such that thse are the (consecutive) points on the runway,
// starting with the "starting" point.
//
// If no such runway, it returns NULL
//
int* airport_runway_to_points(int taxiway);

// Given taxiway 1..26, it returns a list { a, b, c, d,... ,-1}
// such that thse are the (consecutive) points on the taxiway.
//
// If no such taxiway, it returns NULL
//
int* airport_taxiway_to_points(int taxiway);

//=============================================================================

#endif
