
//=============================================================================

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <strings.h>
#include <setjmp.h>
#include <math.h>
#include <unistd.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/time.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/keysym.h>
#include <X11/xpm.h>

#include "gui.h"

//=============================================================================

// The value of PI used for doing angle translation
#define pi ((double)(3.1415926535897932384626433832795))

// The connection to the graphical environment
static Display* local_display=NULL;

// The default depth (typically 16, 24, or 32)
static int local_depth=0;

// The default Bits-per-Pixel (typically 16, 24, or 32)
static int local_bpp=0;

// Colors for the airplanes. (So colors[0] is first, colors[1] is second...)
static unsigned int local_colors[5];

static Window       local_window             ; // The "window" on screen
static Atom         local_atom_delete = None ; // The "window close" message
static int          local_width       =    0 ; // The width of the window
static int          local_height      =    0 ; // The height of the window
static GC           local_gc                 ; // The current line width, etc.
static Pixmap       local_backstore          ; // The drawing buffer
static Pixmap       local_backimage          ; // The airport image
static XFontStruct* local_font_plain  = NULL ; // The current font
static int          local_font_height =    0 ; // The current font height
static int          local_font_width  =    0 ; // The current font width
static int          local_font_ydrop  =    0 ; // The current font's Y-drop

//=============================================================================
//==================== Start/Stop Timer Routine(s) ============================
//=============================================================================

// The timestamp
static struct timeval local_timer;

// This "sets" the timer
static void local_timer_set(void)
 {
  gettimeofday(&local_timer,NULL);
 }

// This "checks" the timer.
// Returns NonZero --> 0.1 second has elapsed. And we've re-activated the timer
// Returns Zero -----> 0.1 second has not yet elapsed.
static int local_timer_check(void)
 {
  long temp; struct timeval timecheck;
  gettimeofday(&timecheck,NULL); temp=timecheck.tv_usec;
  if (timecheck.tv_sec > local_timer.tv_sec+1) { temp += 1000000; }
  if (timecheck.tv_sec > local_timer.tv_sec  ) { temp += 1000000; }
  temp = temp - local_timer.tv_usec;
  if (temp>200000)
   {
    local_timer.tv_sec = timecheck.tv_sec;
    local_timer.tv_usec = timecheck.tv_usec;
    return 1;
   }
  return 0;
 }

//=============================================================================
//===================== Red+Green+Blue -> Color Routine(s) ====================
//=============================================================================

// INPUT  : Red=0..255, Green=0..255, Blue=0..255
// OUTPUT : This computes the color, if it's represented in 16 bits.
//
// (Specifically: the output 16 bits = rrrr rggg gggb bbbb)

#define local_rgb_16(zr,zg,zb) \
 ( ( (((unsigned int)(zr))&0xF8) << 8 ) \
 | ( (((unsigned int)(zg))&0xFC) << 3 ) \
 | ( (((unsigned int)(zb))&0xF8) >> 3 ) )

// INPUT  : Red=0..255, Green=0..255, Blue=0..255
// OUTPUT : This computes the color, if it's represented in 24 or 32 bits.
//
// (Specifically: the output 24/32 bits = rrrr rrrr gggg gggg bbbb bbbb)

#define local_rgb_24or32(zr,zg,zb) \
 ( ( (((unsigned int)(zr))&0xFF) << 16 ) \
 | ( (((unsigned int)(zg))&0xFF) <<  8 ) \
 | ( (((unsigned int)(zb))&0xFF)       ) )

// INPUT  : Red=0..255, Green=0..255, Blue=0..255, BPP=16/24/32
// OUTPUT : This computes the encoded color value.

unsigned int gui_rgb(int zr,int zg,int zb)
 {
  if (local_bpp==16) return local_rgb_16(zr,zg,zb);
  return local_rgb_24or32(zr,zg,zb);
 }

// Here, I redefine "gui_rgb" as a macro, so that in this file,
// we can get shorter code.

#define gui_rgb(zr,zg,zb) ((local_bpp==16) ? \
              (local_rgb_16((zr),(zg),(zb))) : \
          (local_rgb_24or32((zr),(zg),(zb))) )

//=============================================================================
//========================= KeyEvent -> ASCII Code ============================
//=============================================================================

// INPUT: Given a KEY event
//
// OUTPUT:
//   If it's an ordinary key    --> Return the ASCII code for the pressed key
//   All else --------------------> Return 0

static int local_translate_keypress(XEvent* zmsg)
 {
  int zshift = ((zmsg->xkey.state) & ShiftMask) ? 1 : 0;
  int zcap = ((zmsg->xkey.state) & LockMask);
  KeySym zs = XKeycodeToKeysym(local_display,zmsg->xkey.keycode,zshift);

  // Ignore keys that involve the Control Key, etc.
  if (( (ControlMask | Mod1Mask | Mod2Mask | Mod3Mask | Mod4Mask | Mod5Mask)
      & (zmsg->xkey.state) )) return 0;

  // If 'A'..'Z' are pressed...
  if (zs>=XK_A && zs<=XK_Z)
     { if (zcap) return (zs-(XK_A))+'a'; return (zs-(XK_A))+'A'; }

  // If 'a'..'z' are pressed...
  if (zs>=XK_a && zs<=XK_z)
     { if (zcap) return (zs-(XK_a))+'A'; return (zs-(XK_a))+'a'; }

  // If '0'..'9' are pressed...
  if (zs>=XK_0 && zs<=XK_9) return (zs-(XK_0))+'0';
  if (zs>=XK_KP_0 && zs<=XK_KP_9) return (zs-(XK_KP_0))+'0';

  // All else...
  switch(zs)
   {
    case XK_Escape      :                       return  27  ;
    case XK_grave       :                       return '`'  ;
    case XK_asciitilde  :                       return '~'  ;
    case XK_exclam      :                       return '!'  ;
    case XK_at          :                       return '@'  ;
    case XK_numbersign  :                       return '#'  ;
    case XK_dollar      :                       return '$'  ;
    case XK_percent     :                       return '%'  ;
    case XK_asciicircum :                       return '^'  ;
    case XK_ampersand   :                       return '&'  ;
    case XK_asterisk    : case XK_KP_Multiply : return '*'  ;
    case XK_parenleft   :                       return '('  ;
    case XK_parenright  :                       return ')'  ;
    case XK_minus       : case XK_KP_Subtract : return '-'  ;
    case XK_underscore  :                       return '_'  ;
    case XK_equal       :                       return '='  ;
    case XK_plus        : case XK_KP_Add      : return '+'  ;
    case XK_BackSpace   :                       return  8   ;
    case XK_Tab         :                       return '\t' ;
    case XK_bracketleft :                       return '['  ;
    case XK_braceleft   :                       return '{'  ;
    case XK_bracketright:                       return ']'  ;
    case XK_braceright  :                       return '}'  ;
    case XK_backslash   :                       return '\\' ;
    case XK_bar         :                       return '|'  ;
    case XK_semicolon   :                       return ';'  ;
    case XK_colon       :                       return ':'  ;
    case XK_apostrophe  :                       return '\'' ;
    case XK_quotedbl    :                       return '\"' ;
    case XK_Return      : case XK_KP_Enter    : return  13  ;
    case XK_comma       :                       return ','  ;
    case XK_less        :                       return '<'  ;
    case XK_period      : case XK_KP_Decimal  : return '.'  ;
    case XK_greater     :                       return '>'  ;
    case XK_slash       : case XK_KP_Divide   : return '/'  ;
    case XK_question    :                       return '?'  ;
    case XK_space       :                       return ' '  ;
   }

  return 0; // Return 0 for a un-recognized key.
 }

//=============================================================================
//====================== Airplane Drawing Routine(s) ==========================
//=============================================================================

// The coordinates for a right-facing airplane, that is centered at (50,50)
static XPoint local_plane[21] =
 { {99,49} , {84,42} , {64,36} , {54, 0} , {39, 5} , {44,33} , {46,41}
 , {44,33} , {32,31} , {31,40} , {39,49} , {31,59} , {32,68} , {39,66}
 , {46,58} , {44,66} , {39,95} , {54,99} , {64,63} , {84,57} , {99,49} };

// Undraw an airplane that was drawn at (dx,dy)
void gui_undrawPlane(int dx,int dy)
 {
  int dw=80,dh=50;// Width+Height. Must be big enough to cover the whole plane.
  if (dx<(dw/2)) { dw=dw-dx; dx=0; } else { dx=dx-(dw/2); }
  if (dy<(dh/2)) { dh=dh-dy; dy=0; } else { dy=dy-(dh/2); }
  if (local_display==NULL) return;
  XCopyArea(local_display,local_backimage,local_backstore
    ,local_gc,dx,dy,dw,dh,dx,dy);
 }

// Draw an airplane at (dx,dy), rotated at "deg" degrees, with color=1..
void gui_drawPlane(int dx,int dy,int deg,int color)
 {
  int i;
  double dd=(((double)deg) / 360) * 2 * pi; // The degree (in radian)
  double tl=cos(dd); double bl=sin(dd); // sine/cosine used for doing rotation
  XPoint temp[21]; // Buffer to store the "rotated" airplane image
  for(i=0; i<21; i=i+1)
   {
    // Compute the rotated airplane image
    double x=(local_plane[i].x-50)*100;
    double y=(local_plane[i].y-50)*100;
    temp[i].x = ((int) ((x*tl-y*bl)/400)) + dx;
    temp[i].y = ((int) ((x*bl+y*tl)/400)) + dy;
   }
  if (local_display==NULL) return;
  color=(color+4)%5; if (color<0) color=0;
  // Draw the INSIDE of the plane
  XSetLineAttributes(local_display,local_gc,1,LineSolid,CapRound,JoinRound);
  XSetForeground(local_display,local_gc,local_colors[color]);
  XFillPolygon(local_display,local_backstore,local_gc
    ,temp,21,Nonconvex,CoordModeOrigin);
  // Draw the BORDER of the plane
  XSetLineAttributes(local_display,local_gc,2,LineSolid,CapRound,JoinRound);
  XSetForeground(local_display,local_gc,gui_rgb(0x30,0x30,0x30));
  XDrawLines(local_display,local_backstore,local_gc,temp,21,CoordModeOrigin);
 }

//=============================================================================
//========================= Text Drawing Routine(s) ===========================
//=============================================================================

static char local_textbuf[gui_textwidth*gui_textheight]; // The text buffer
static int local_textX=0;       // Current cursor location (X=0..width-1)
static int local_textY=0;       // Current cursor location (Y=0..height-1)
static int local_textLeft=0;    // The # of pixels to use as LEFT MARGIN

// This function scrolls the Text Window up by 1 line
static void local_text_ScrollUp(void)
 {
  memmove(local_textbuf,local_textbuf+gui_textwidth
    ,sizeof(local_textbuf)-(gui_textwidth));
  memset(local_textbuf+(gui_textheight-1)*gui_textwidth
    ,' ',gui_textwidth);
  XCopyArea(local_display,local_backstore,local_backstore,local_gc
    ,local_textLeft,local_font_height
    ,(local_width-local_textLeft+1),local_height-local_font_height
    ,local_textLeft,0);
  XSetLineAttributes(local_display,local_gc,1,LineSolid,CapRound,JoinRound);
  XSetForeground(local_display,local_gc,gui_rgb(255,255,255));
  XFillRectangle(local_display,local_backstore,local_gc
    ,local_textLeft,local_height-local_font_height
    ,local_width-(local_textLeft)
    ,local_font_height*2
    );
  XSetForeground(local_display,local_gc,0);
 }

// If show==0 --> This hides the cursor
// If show!=0 --> This shows the cursor
static void local_text_ShowHideCursor(int show)
 {
  XSetLineAttributes(local_display,local_gc,2,LineSolid,CapRound,JoinRound);
  if (!show) XSetForeground(local_display,local_gc,gui_rgb(255,255,255));
     else XSetForeground(local_display,local_gc,0);
  XFillRectangle(local_display,local_backstore,local_gc
    ,local_textLeft+(local_textX*local_font_width)+1
    ,(local_textY)*(local_font_height)+2
    ,local_font_width-1, local_font_height-2);
  XSetForeground(local_display,local_gc,0);
 }

// Erase the last drawn character, and move the cursor backward.
void gui_text_delchar(void)
 {
  char zc; if (local_textX==0 && local_textY==0) return;
  local_text_ShowHideCursor(0);
  local_textX--;
  if (local_textX<0) { local_textX=gui_textwidth-1; local_textY--; }
  // Erase the character from the memory buffer
  zc=local_textbuf[local_textY*gui_textwidth + local_textX];
  local_textbuf[local_textY*gui_textwidth + local_textX]=' ';
  // Erase the character from the screen
  XSetForeground(local_display,local_gc,gui_rgb(255,255,255));
  XDrawString(local_display,local_backstore,local_gc
    ,local_textLeft+(local_textX*local_font_width)
    ,(local_textY*local_font_height)+(local_font_ydrop),&zc,1);
  XSetForeground(local_display,local_gc,0);
  local_text_ShowHideCursor(1);
 }

// Draw 1 character onto the text window
// Note: The color is specifed via the parameter "zcolor"
// Note: If it's the New Line character, then the textwindow
//       will insert a blank line.
void gui_text_addchar(char zc,unsigned int zcolor)
 {
  if (local_display==NULL) return;
  local_text_ShowHideCursor(0);
  if (zc>=32 && zc<=126)
   {
    if (zcolor!=0) XSetForeground(local_display,local_gc,zcolor);
    XDrawString(local_display,local_backstore,local_gc
      ,local_textLeft+(local_textX*local_font_width)
      ,(local_textY*local_font_height)+(local_font_ydrop),&zc,1);
    XSetForeground(local_display,local_gc,0);
    local_textbuf[local_textY*gui_textwidth + local_textX] = zc;
    local_textX++;
    if (local_textX>=gui_textwidth) { local_textX=0; local_textY++; }
   }
  else
   {
    local_textX=0; local_textY++;
   }
  if (local_textY>=gui_textheight) { local_textY--; local_text_ScrollUp(); }
  local_text_ShowHideCursor(1);
 }

// Draw an entire block of characters onto the text window.
// Note: The color is specifed via the parameter "zcolor"
// Note: If (zc) contains the New Line character,
//       then the appropriate Blank Line will be drawn on the window.
void gui_text_addstring(char* zc,unsigned int zcolor)
 {
  char *zscan; int zn;
  if (local_display==NULL) return;
  local_text_ShowHideCursor(0);
  while(1)
   {
    if (*zc==0) { local_text_ShowHideCursor(1); return; }
    zn=0;
    zscan=zc;
    while((*zscan)>=32 && (*zscan)<=126)
     {
      zscan++; zn++; if (zn+local_textX>=gui_textwidth) break;
     }
    if (zn)
     {
      if (zcolor!=0) XSetForeground(local_display,local_gc,zcolor);
      XDrawString(local_display,local_backstore,local_gc
        ,local_textLeft+(local_textX*local_font_width)
        ,(local_textY*local_font_height)+(local_font_ydrop),zc,zn);
      XSetForeground(local_display,local_gc,0);
      memcpy(local_textbuf + local_textY*gui_textwidth + local_textX,zc,zn);
      zc=zc+zn; local_textX=local_textX+zn;
     }
    if (*zc==10 || *zc==13)
     {
      local_textX=0; local_textY++;
      if (zc[1]==(10+13-zc[0])) zc=zc+1;
      zc=zc+1;
     }
    if (local_textX>=gui_textwidth) { local_textX=0; local_textY++; }
    if (local_textY>=gui_textheight) { local_textY--; local_text_ScrollUp(); }
   }
 }

//=============================================================================

// This function makes sure the delayed graphics do get drawn on screen
void gui_commit(void)
 {
  if (local_display==NULL) return;
  XCopyArea(local_display,local_backstore
     ,local_window,local_gc,0,0,local_width,local_height,0,0);
 }

//=============================================================================

// This function displays the "graphical window" on the screen.
//
// INPUT:
//   zcname      = "class name" for the window
//   zwname      = "window title"
//   (my_x,my_y) = The top-left corner for the window
//
// Note: It will open "background.xpm" and display it as
// the background of the window.

void gui_OpenWindow(char* zcname,char *zwname,int my_x,int my_y)
 {
  XSetWindowAttributes zattr;
  XSizeHints zsize;
  XClassHint zclass;
  XGCValues zvalues;
  Screen* local_screen;
  XpmImage my_image; // the background image
  XpmInfo info; // info on the background image
  int my_w; // Window width
  int my_h; // Window height

  // Open a connection to the X server
  local_display=XOpenDisplay(NULL); if (local_display==NULL) goto bad;

  // Pick the default Screen
  local_screen=DefaultScreenOfDisplay(local_display);

  // Pick the default Depth
  local_depth=DefaultDepthOfScreen(DefaultScreenOfDisplay(local_display));

  // Determine the best Bits Per Pixel (Reject if it's not 16 or 24 or 32)
  local_bpp=0;
  {
   int zi; XPixmapFormatValues* zlist=XListPixmapFormats(local_display,&zi);
   if (zlist==NULL) goto bad;
   for(zi=zi-1;zi>=0;zi=zi-1)
    if (zlist[zi].depth == local_depth)
     {
      int zb = zlist[zi].bits_per_pixel;
      if (zb>local_bpp) if (zb==16 || zb==24 || zb==32) local_bpp=zb;
     }
   XFree(zlist);
   if (local_bpp==0)
    {
     printf("ERROR: The bits-per-pixel must be 16, 24, or 32.\n");
     exit(1);
    }
  }

  // Load the background image...
  if (XpmReadFileToXpmImage("background.xpm",&my_image,&info)!=XpmSuccess)
   {
    printf("ERROR: Fail to load \"background.xpm\"!"); exit(1);
   }

  // Load the default font
  local_font_plain  = XLoadQueryFont(local_display,"fixed");
  local_font_ydrop  = local_font_plain->ascent;
  local_font_height = (local_font_plain->ascent + local_font_plain->descent);
  local_font_width  = (local_font_plain->max_bounds.width);

  // Determine the WindowWidth and WindowHeight and "local_textLeft"
  local_textLeft = my_image.width + 4;
  my_h = my_image.height;
  my_w = local_textLeft + (local_font_width * gui_textwidth);
  if ((my_w & 3)!=0) my_w = ((my_w | 3)+1); // Make "width" be a multiple of 4
  local_width=my_w;
  local_height=my_h;

  // Create the window
  bzero(&zattr,sizeof(zattr));
  zattr.background_pixel  = WhitePixelOfScreen(local_screen);
  zattr.backing_store     = 0;
  zattr.override_redirect = 0;
  local_window = XCreateWindow(local_display, RootWindowOfScreen(local_screen)
     ,my_x ,my_y, my_w, my_h, 0, CopyFromParent, InputOutput, CopyFromParent
     ,CWBackingStore | CWBackPixel | CWOverrideRedirect, &zattr);

  // Set the SIZE
  bzero(&zsize,sizeof(zsize));
  zsize.x=(my_x); zsize.y=(my_y); zsize.flags = USPosition | USSize;
  XSetWMNormalHints(local_display, local_window, &zsize);

  // Set the Class Hint
  bzero(&zclass,sizeof(zclass));
  zclass.res_name=zcname; zclass.res_class=zcname;
  XSetClassHint(local_display, local_window, &zclass);

  // Set the window title
  XStoreName(local_display, local_window, zwname);

  // Enable the "Window Close" message
  local_atom_delete = XInternAtom(local_display,"WM_DELETE_WINDOW",0);
  if (local_atom_delete==None) goto bad;
  XSetWMProtocols(local_display,local_window,&local_atom_delete,1);

  // Choose the set of X events we care about...
  XSelectInput(local_display, local_window, ExposureMask | StructureNotifyMask
    | KeyPressMask | KeyReleaseMask | ButtonPressMask | ButtonReleaseMask
    | PointerMotionMask);

  // Load the default colors
  local_colors[0] = gui_rgb(0xFF,0x00,0x00); // Red
  local_colors[1] = gui_rgb(0xFF,0x90,0x50); // Orange
  local_colors[2] = gui_rgb(0xFF,0xFF,0x60); // Yellow
  local_colors[3] = gui_rgb(0x30,0xFF,0x30); // Green
  local_colors[4] = gui_rgb(0x30,0x30,0xFF); // Blue

  // Create the Graphics Context
  bzero(&zvalues,sizeof(zvalues)); zvalues.graphics_exposures=0;
  local_gc = XCreateGC(local_display,local_window
    ,GCGraphicsExposures,&zvalues);
  XSetFont(local_display, local_gc,local_font_plain->fid);
  XSetForeground(local_display, local_gc, WhitePixelOfScreen(local_screen));
  XSetBackground(local_display, local_gc, BlackPixelOfScreen(local_screen));

  // Set the "dashed" line style
  {char zdash[]={4,9}; XSetDashes(local_display,local_gc,0,zdash,2);}

  // Create the background image buffer "local_backimage"
  XpmCreatePixmapFromXpmImage(local_display,local_window,&my_image
    ,&local_backimage,NULL,NULL);

  // Clear the text buffer
  memset(local_textbuf,' ',sizeof(local_textbuf));

  // Create the graphical buffer
  local_backstore=XCreatePixmap(local_display,local_window
    ,WidthOfScreen(local_screen),HeightOfScreen(local_screen),local_depth);
  XFillRectangle(local_display,local_backstore,local_gc,0,0,my_w,my_h);
  XCopyArea(local_display,local_backimage,local_backstore
    ,local_gc,0,0,my_w,my_h,0,0);

  // Start the "timeout" timer...
  local_timer_set();

  // Success --> Show the window!
  XMapWindow(local_display,local_window); return;

  // Failure --> Display an error, then end the program.
  bad: printf("ERROR: Graphical environment initialization failed!\n");
  exit(1);
 }

//=============================================================================

// This function checks to see if there are any KeyPress events.
// (At the same time, it calls "local_timer_check" to see if we've waited
// too long for a KeyPress. If we have waited too long, we'll return 0.)
//
// Optionally, you can pass in a "file descriptor" for it to inspect.
// If either a KeyPress happened, or if the "file" received new readable data,
// then the function will return immediately without waiting for a timeout.
// (If this feature is not needed, pass in "-1")
//
// If "timeout" => Returns 0
// If user pressed an ordinary key => Return the key
// All else => Return 0
//
// (Note: If the key is printable, it will be between 32..126 inclusively.
// If the key is ENTER, the return value is 13.
// If the key is ESC, the return value is 27.)

int gui_HandleEvents(int file)
 {
  if (local_display==NULL) return 0;
  while(1)
   {
    XEvent zmsg;
    if (XCheckMaskEvent(local_display,0xFFFFFFFF,&zmsg)==0)
     {
      struct timeval alltime;
      fd_set allset; int maxfile,file2=(ConnectionNumber(local_display));
      alltime.tv_sec=0; alltime.tv_usec=200000;
      FD_ZERO(&allset); FD_SET(file2,&allset);
      if (file>=0) FD_SET(file,&allset);
      if (file>file2) maxfile=file+1; else maxfile=file2+1;
      while(1)
       {
        maxfile=select(maxfile,&allset,NULL,NULL,&alltime);
        if (maxfile>0) break;
        if (maxfile==0) { local_timer_set(); return 0; }
        if (errno!=EINTR && errno!=EAGAIN && errno!=EWOULDBLOCK) exit(1);
       }
      if (!FD_ISSET(file2,&allset)) return 0;
      if (local_timer_check()) return 0;
      XNextEvent(local_display,&zmsg);
     }
    if (zmsg.type==Expose)
     {
      int x=zmsg.xexpose.x; int w=zmsg.xexpose.width;
      int y=zmsg.xexpose.y; int h=zmsg.xexpose.height;
      if (local_window != (zmsg.xexpose.window)) return 0;
      if (x+w-1 >= local_width ) w=local_width-x;
      if (y+h-1 >= local_height) h=local_height-y;
      XSetForeground(local_display,local_gc,gui_rgb(255,255,255));
      XCopyArea(local_display,local_backstore
        ,local_window,local_gc,x,y,w,h,x,y);
      w=zmsg.xexpose.width; h=zmsg.xexpose.height;
      if (x+w-1 >= local_width)
       {
        int excess=(x+w-local_width); int newx=(x+w-excess);
        XFillRectangle(local_display,local_window,local_gc,newx,y,excess,h);
       }
      if (y+h-1 >= local_height)
       {
        int excess=(y+h-local_height); int newy=(y+h-excess);
        XFillRectangle(local_display,local_window,local_gc,x,newy,w,excess);
       }
      return 0;
     }
    if (zmsg.type==KeyPress)
     {
      int key = local_translate_keypress(&zmsg);
      if (local_window != (zmsg.xkey.window)) return 0;
      if (key==27 || key==8 || key==10 || key==13) return key;
      if (key>=32 && key<=126) return key;
      return 0;
     }
    if (zmsg.type==ButtonPress)
     {
      char buf[1024]; int zx=(zmsg.xbutton.x),zy=(zmsg.xbutton.y);
      sprintf(buf,"(%d,%d)",zx,zy);
      XSetLineAttributes(local_display,local_gc
         ,2,LineSolid,CapRound,JoinRound);
      XSetForeground(local_display,local_gc,gui_rgb(0xFF,0xFF,0xFF));
      XFillRectangle(local_display,local_backstore,local_gc
         ,10,10,15*local_font_width,local_font_height);
      XSetForeground(local_display,local_gc,gui_rgb(0xFF,0xA0,0xA0));
      XDrawString(local_display,local_backstore,local_gc,10,10+local_font_ydrop
         ,buf,strlen(buf));
      return 0;
     }
    if (zmsg.type==ClientMessage)
     {
      if (local_window != (zmsg.xkey.window)) return 0;
      if (((Atom)(zmsg.xclient.data.l[0]))==local_atom_delete) return 27;
      return 0;
     }
   }
 }

//=============================================================================

// This function closes the "graphical window"

void gui_CloseWindow(void)
 {
  if (local_display)
   {
    XDestroyWindow(local_display,local_window);
    XCloseDisplay(local_display);
   }
 }

//=============================================================================
