
#ifndef include_gui_h
#define include_gui_h

//=============================================================================
//**** Simple Usage Scenario **************************************************
//
//   gui_OpenWindow("aircon","Air Traffic Control Simulator",0,0);
//   while(1)
//    {
//     int x = gui_HandleEvents(-1);
//     if (x=='A' || x=='a')
//      {
//       gui_text_addstring("Hello!\n" , gui_rgb(255,0,0)); // Draws a string
//       gui_text_addchar('a' , gui_rgb(255,0,0));         // Draws 1 character
//       gui_commit();                          // Commits the above operations
//       continue;
//      }
//     if (x=='B' || x=='b')
//      {
//       gui_drawPlane(20,20,100,gui_rgb(255,255,0)); // Draws plane at (20,20)
//       gui_commit();                          // Commits the above operations
//      }
//     if (x==27) break;
//    }
//   gui_CloseWindow();
//
//**** Below are the gory details of the various functions mentioned above ****
//=============================================================================

#define gui_textwidth  66 // Width  of the "text window"
#define gui_textheight 54 // Height of the "text window"

//=============================================================================

// This function displays the "graphical window" on the screen.
//
// INPUT:
//   zcname      = "class name" for the window
//   zwname      = "window title"
//   (my_x,my_y) = The top-left corner for the window
//
// Note: It will open "background.xpm" and show it as the window's background.
//
void gui_OpenWindow(char* zcname,char *zwname,int x,int y);

//=============================================================================

// This function checks to see if there are any KeyPress events.
// (At the same time, it calls "timer_check" to see if we've waited
// too long for a KeyPress. If we have waited too long, we'll return 0.)
//
// Optionally, you can pass in a "file descriptor" for it to inspect.
// If either a KeyPress happened, or if the "file" received new readable data,
// then the function will return immediately without waiting for a timeout.
// (If this feature is not needed, pass in "-1")
//
// If "timeout" => Returns 0
// If user pressed an ordinary key => Return the key
// All else => Return 0
//
// (Note: If the key is printable, it will be between 32..126 inclusively.
// If the key is ENTER, the return value is 13.
// If the key is ESC, the return value is 27.)

int gui_HandleEvents(int file);

//=============================================================================

// This function closes the "graphical window"
//
void gui_CloseWindow(void);

//=============================================================================

// Draw 1 character onto the text window
// Note: The color is specifed via the parameter "zcolor"
// Note: If it's the New Line character, then the textwindow
//       will insert a blank line.
void gui_text_addchar(char zc,unsigned int zcolor);

// Draw an entire block of characters onto the text window.
// Note: The color is specifed via the parameter "zcolor"
// Note: If (zc) contains the New Line character,
//       then the appropriate Blank Line will be drawn on the window.
void gui_text_addstring(char* zc,unsigned int zcolor);

// Erase the last drawn character, and move the cursor backward.
void gui_text_delchar(void);

//=============================================================================

// Draw an airplane at (dx,dy), rotated at "deg" degrees, with color=1...
void gui_drawPlane(int dx,int dy,int deg,int color);

// Erase an airplane that was previously drawn at (dx,dy)
void gui_undrawPlane(int dx,int dy);

//=============================================================================

// Note: The above "gui_text" functions, and the "draw/undraw/move plane"
// functions do not immediately show up on the screen.
//
// Call this function (after the gui_text... and plane... functions)
// to make sure the graphics are updated on the screen.
//
void gui_commit(void);

//=============================================================================

// INPUT  : Red=0..255, Green=0..255, Blue=0..255.
// OUTPUT : This computes the encoded color value.
unsigned int gui_rgb(int zr,int zg,int zb);

//=============================================================================

#endif
