
#ifndef include_lisp_h
#define include_lisp_h

//=============================================================================

extern char buffer1[16384];   // Must be >= 100 bytes. Preferably much bigger.
extern char buffer2[16384*2]; // Must be at least twice the size of "buffer1".

//=============================================================================

// This function initializes the connection to the LISP parser by...
// (1) Execute the command "mlisp -s air_control.lisp" in the background
// (2) Return the connection "return_in"  by which we can READ from LISP
// (3) Return the connection "return_out" by which we can WRITE to LISP
//
// If failure -> Return 0
// If success -> Return 1 (And "*return_in" and "*return_out" will contain
//                         the pipe that we can use to read/write to LISP)

int lisp_init(int* return_in, int* return_out);

//=============================================================================

// Given "file" is a connection to read from LISP.
// It tries to read a block of data from the LISP program into "buffer1".
// (Note: If LISP hasn't said anything, then it immediately returns 0)
//
// Returns == 0  -> LISP hasn't generated enough output. Please try again soon.
// Returns == 1  -> LISP is newly initialized. And LISP is ready for input.
// Returns == 2  -> LISP is ready for input.
//
// Note: Whenever this function has accumulated enough LISP output,
// it will send them to the discourse manager.

int lisp_read(int f);

//=============================================================================

// > Given a line of "user sentence" in "buffer1".
//
// > Given a "planeID=1..." that signifies which plane I'm talking to...
//
// This function cleans it up a bit, store the new sentence in "buffer2",
// then send the contents of "buffer2" to LISP
//
// (Note: This LISP module has no concept of airplanes, simulator, or
// anything like that. The "planeID" is merely used to tag the lines,
// so that later when it passes parser output to the discourse manager,
// it can tell the discourse manager about the "original intended recipient".

void lisp_write(int f,int planeID);

//=============================================================================

#endif
