;;; -*- mode: lisp; package: user; syntax: common-lisp; base: 10 -*-
;;; match-allowing-extras.lisp
;;; public functions: match-allowing-extras, instantiate, combine,
;;;   make-binding-list, 
;;;   uniqueify
;;; nb nil is a binding list that represents failure of a match

;this is the wrong order, but lucid gives a load warning if in-package isn't
;the first line of a file.
; (in-package 'gpsg)

(provide 'match-allowing-extras)

(defun match-allowing-extras (p d &optional (b (make-binding-list)))
  (let ((b (internal-match p d b)))
    (values b (not (equal b (failed-binding-list))))))

(defun internal-match (p d b)
  (cond ((binding-list-failure? b) b)
        ((or (atom p) (atom d))
         (match-pred p d b))
        (t (loop (setf b (match-pred (car p) (car d) b))
		 (unless b (return nil))
		 (setf p (cdr p))
		 (setf d (cdr d))
		 (cond ((variable? p) ;; the (xxx . ?v) case
			(return (match-pred p d b)))
		       ((or (null p) (null d))
;;;			(if (and (null p) (null d)) ;change for match-allowing-extras
			(if (null p)
			    (return b)
			  (return (failed-binding-list)))))))))

(defun match-pred (p d b)
  (cond ((equal p d)
         b)
        ((and (variable? p) (bound? p b))
         (internal-match (lookup-variable p b) d b))
        ((and (variable? d) (bound? d b))
         (internal-match p (lookup-variable d b) b))
        ((and (variable? p) (freefor? d p b))
         (let ((new-b (copy-binding-list b)))
           (bind-variable p d new-b)
	   new-b))
        ((and (variable? d) (freefor? p d b))
         (let ((new-b (copy-binding-list b)))
           (bind-variable d p new-b)
	   new-b))
        (t
         (if (or (atom p) (atom d))
             (if (equal p d)
                 b
                 (failed-binding-list))
             (internal-match p d b)))))

(defun instantiate (p b)
  (cond ((null p) nil)
        ((atom p) 
         (cond ((variable? p)                   
                (let ((p-value (lookup-variable p b)))
                  (cond ((null p-value) p)
                        ((variable? p-value)
                         (instantiate p-value b))
                        (t p-value))))
               (t p)))          
        (t (cons (instantiate (car p) b)
                 (instantiate (cdr p) b)))))

(defun freefor? (exp var b)
  (cond ((variable? exp)
         (cond ((equal exp var) nil)
               (t (freefor? (lookup-variable exp b) var b))))
        ((atom exp) t)
        (t (and (freefor? (car exp) var b)
                (freefor? (cdr exp) var b)))))

;;; binding lists

(defun print-binding-list (binding-list stream print-level)
  (declare (ignore print-level))
  (format stream "{")
  (mapc #'(lambda (pair) (format stream "~s~s " (car pair) (cdr pair)))
	(binding-list-bindings binding-list))
  (format stream "}"))

(defstruct (binding-list (:print-function print-binding-list) (:copier nil)) bindings)

(defun bound? (variable binding-list)
  (assoc variable (binding-list-bindings binding-list)))

(defun lookup-variable (variable binding-list)
  (let ((bucket (assoc variable (binding-list-bindings binding-list))))
    (if bucket (cdr bucket))))

(defun bind-variable (variable value binding-list)
  (push (cons variable value) (binding-list-bindings binding-list)))

(defun binding-list-domain (binding-list)
  (mapcar #'car (binding-list-bindings binding-list)))

(defun binding-list-failure? (binding-list)
  (null binding-list))

(defun copy-binding-list (binding-list)
  (make-binding-list :bindings
		     (copy-alist (binding-list-bindings binding-list))))

(defun failed-binding-list ()
  nil)

(defun valid-env? (env) env) ;; 4-july-88

(defun combine (bl1 bl2) ;; unsure about this fcn
  (let ((bl3 (copy-binding-list bl1))
        (vars-in-bl1 (binding-list-domain bl1)))
    (block match-up
      (dolist (var-value (binding-list-bindings bl2)) ;; cvted to cl 4-jul-88
	(let ((var (car var-value))
	      (value (cdr var-value)))
	  (cond ((member var vars-in-bl1 :test #'equal)
		 (unless (equal (lookup-variable var bl1)
				(lookup-variable var bl2))
			 (return-from match-up (failed-binding-list))))
		(t (bind-variable var value bl3)))))
      bl3)))

;;; variables

(defun variable? (x)
  (and (symbolp x)
       (char= (char (symbol-name x) 0) #\?)))

(defun add-suffix-number-to-variable (variable suffix-number)
  (intern (coerce (append (coerce (symbol-name variable) 'list)
			  (coerce (format nil "-~s" suffix-number) 'list))
		  'string)))

;;; hack to get unique variable names

(defvar *uniqueify-instance-count* 0)

(defun reset-instance-count ()
  (declare (special *uniqueify-instance-count*))
  (setq *uniqueify-instance-count* 0))

(defun uniqueify (exp)
  (declare (special *uniqueify-instance-count*))
  (incf *uniqueify-instance-count*)
  (add-suffix-number-to-variables-in-tree exp *uniqueify-instance-count*))

(defun add-suffix-number-to-variables-in-tree (exp suffix)
  (cond ((variable? exp)
         (add-suffix-number-to-variable exp suffix))
        ((consp exp)
         (cons (add-suffix-number-to-variables-in-tree (car exp) suffix) 
               (add-suffix-number-to-variables-in-tree (cdr exp) suffix)))
        (t exp)))

;;; rules

(defun print-rule (rule stream print-level)
  (declare (ignore print-level))
  (format stream "[~s=>~s/~s"
	  (rule-lhs rule)
	  (rule-rhs rule)
	  (rule-bindings rule))
  (cond ((eq (rule-type rule) 'normal)
	 (format stream "]"))
	(t (format stream " ~s]" (rule-type rule)))))

;;;(defstruct (rule (:print-function print-rule))
(defstruct rule ;; without the print-function 11-may-88
  lhs 
  rhs
  (bindings (make-binding-list))
  (type 'normal)
  (priority 1))

(defvar *sample-rule-set-1* (list (make-rule :lhs '((if ?a ?b) ?a) :rhs '?b)
				(make-rule :lhs '((or ?a ?b) (not ?a)) :rhs '?b)
				(make-rule :lhs '((or ?a ?b) (not ?b)) :rhs '?a)
				(make-rule :lhs '((not (not ?a))) :rhs '?a)
				(make-rule :lhs '((and ?a ?b)) :rhs '?a)
				(make-rule :lhs '((and ?a ?b)) :rhs '?b)))

;;; added 4-july-88

(defun make-env () (make-binding-list))

(defun same-env? (e1 e2)
  (let ((dom1 (binding-list-domain e1))
	(dom2 (binding-list-domain e2)))
    (and (= (length dom1) (length dom2))
	 (every #'(lambda (variable)
		    (equal (lookup-variable variable e1)
			   (lookup-variable variable e2)))
		dom1))))

(defun same-pattern? (p1 p2)
  (equal p1 p2))
		    
(defstruct conflict-set-element rule env datum)

(defun filter (pred seq) (remove-if-not pred seq))

(defun combine-env (env1 env2) (combine env1 env2))


