
;==============================================================================

; Leading numbers must add "$"
;     come at 5 PM
;  => come at $5 PM

; All single quote ' must be changed to ^
;     come at 5 o'clock
;  => come at 5 o^clock

;==============================================================================

(setf *show-syntax?* nil *show-semantics?* nil)

(defun list-append (list1 list2)
  (if (null list1)
      list2
      (cons (car list1) (list-append (cdr list1) list2))
  ))

(defun has-feature (frame feature)
  (if (null frame)
      nil
      (has-feature-helper (cdr frame) feature)
  ))

(defun has-feature-helper (frame feature)
  (if (null frame)
      nil
      (if (eq (car frame) feature)
          (eq 1 1)
          (has-feature-helper (cdr (cdr frame)) feature)
      )))

(defun my-add-feature (list1 name1 value1)
  (cons (car list1)
        (my-add-feature-helper (cdr list1) name1 value1)))

(defun my-add-feature-helper (list1 name1 value1)
  (if (null list1)
      (cons name1 (cons value1 nil))
      (if (string< (car list1) name1)
          (cons (car list1)
                (cons (car (cdr list1))
                   (my-add-feature-helper (cdr (cdr list1)) name1 value1)))
          (if (string> (car list1) name1)
              (cons name1 (cons value1 list1))
              (cons name1 (cons value1 list1))
          )
      )))

(defun my-addnew-feature (list1 name1 value1)
  (cons (car list1)
        (my-addnew-feature-helper (cdr list1) name1 value1)))

(defun my-addnew-feature-helper (list1 name1 value1)
  (if (null list1)
      (cons name1 (cons value1 nil))
      (if (string< (car list1) name1)
          (cons (car list1)
                (cons (car (cdr list1))
                   (my-addnew-feature-helper (cdr (cdr list1)) name1 value1)))
          (if (string> (car list1) name1)
              (cons name1 (cons value1 list1))
              list1
          )
      )))

(defun my-addprepend-feature (list1 name1 value1)
  (cons (car list1)
        (my-addprepend-feature-helper (cdr list1) name1 (cons value1 nil))))

(defun my-addprepend-feature-helper (list1 name1 value1)
  (if (null list1)
      (cons name1 (cons value1 nil))
      (if (string< (car list1) name1)
          (cons (car list1)
                (cons (car (cdr list1))
                 (my-addprepend-feature-helper (cdr (cdr list1)) name1 value1)))
          (if (string> (car list1) name1)
              (cons name1 (cons value1 list1))
              (cons name1 (cons (list-append value1 (car (cdr list1))) (cdr (cdr list1))))
          )
      )))

(defun print-template (sentence) (format t "[OUTPUT] ~s~%" sentence))

;==============================================================================

(add-rule-sem '(root ==> s      )  '(lambda (s) t))
(add-rule-sem '(root ==> s and s)  '(lambda (s1 and s2) t))

;==============================================================================

; (Default)
(add-rule-sem '(s ==> pronoun vp)
   '(lambda (a1 a2) (funcall a2 a1)))

; When the pronoun is skipped, this is a command to "you".
; So the agent is "you".
(add-rule-sem '(s ==> vp)
   '(lambda (a1) (funcall a1 'you)))

; Replace "clear to <vp>"                      with "you <vp>"
; Replace "here is what i want you to do <vp>" with "you <vp>"
; Replace "i am going to have you <vp>"        with "you <vp>"
; Replace "i need/want you to <vp>"            with "you <vp>"
; Replace "you are going to <vp>"              with "you <vp>"
; Replace "you are going to have to <vp>"      with "you <vp>"
; Replace "you want to <vp>"                   with "you <vp>"
(add-rule-sem '(s ==> clear to vp)
   '(lambda (a1 a2 a3) (funcall a3 'you)))
(add-rule-sem '(s ==> here is what i want you to do vp)
   '(lambda (a1 a2 a3 a4 a5 a6 a7 a8 a9 ) (funcall a9 'you)))
(add-rule-sem '(s ==> i am going to have you vp)
   '(lambda (a1 a2 a3 a4 a5 a6 a7) (funcall a7 'you)))
(add-rule-sem '(s ==> i need you to vp)
   '(lambda (a1 a2 a3 a4 a5) (funcall a5 'you)))
(add-rule-sem '(s ==> i want you to vp)
   '(lambda (a1 a2 a3 a4 a5) (funcall a5 'you)))
(add-rule-sem '(s ==> you are going to vp)
   '(lambda (a1 a2 a3 a4 a5) (funcall a5 'you)))
(add-rule-sem '(s ==> you are going to have to vp)
   '(lambda (a1 a2 a3 a4 a5 a6 a7) (funcall a7 'you)))
(add-rule-sem '(s ==> you want to vp)
   '(lambda (a1 a2 a3 a4) (funcall a4 'you)))

;==============================================================================

; (Default)
(add-rule-sem '(vp ==> v+args) #'identity)

; Replace "be V-ing" -> "V"
(add-rule-sem '(vp ==> be-verb   vp) '(lambda (a1 a2   ) a2))

; Replace "clear for V-ing" -> "V"
(add-rule-sem '(vp ==> clear for vp) '(lambda (a1 a2 a3) a3))

;==============================================================================

;altimeter <altimeter value>
(add-rule-sem '(v+args ==> altimeter altimeter-value)
  '(lambda (a av) `(lambda (subj) (print-template
      `(altimeter :value ,',av :agent ,subj)))))

;cancel that
;change of plans
(add-rule-sem '(v+args ==> cancel that)
  '(lambda (a b) `(lambda (subj) (print-template
      `(undo :agent ,subj)))))
(add-rule-sem '(v+args ==> change of plan)
  '(lambda (a b c) `(lambda (subj) (print-template
      `(undo :agent ,subj)))))

;change speed to <speed value>
(add-rule-sem '(v+args ==> change speed to speed-value)
  '(lambda (a b c s) `(lambda (subj) (print-template
      `(setspeed :goal ,',s :agent ,subj)))))

;checkin at <num> o'clock
;checkin in <num> minutes
(add-rule-sem '(v+args ==> checkin time-value)
  '(lambda (c tv) `(lambda (subj) (print-template
      `(checkin :time ,',tv :agent ,subj)))))

;intercept <num>
(add-rule-sem '(v+args ==> intercept some-number)
  '(lambda (a1 a2) `(lambda (subj) (print-template
      `(intercept :patient ,',a2 :agent ,subj)))))

; climb   and maintain <altitude value>
; descend and maintain <altitude value>
; [on departure] maintain <altitude value> [at departure]
; maintain heading of <num>
; maintain this frequency
(add-rule-sem '(v+args ==> climb and maintain altitude-value)
  '(lambda (a1 a2 a3 a4) `(lambda (subj) (print-template
      `(climb-maintain :alt ,',a4 :agent ,subj))))
  )
(add-rule-sem '(v+args ==> descend and maintain altitude-value)
  '(lambda (a1 a2 a3 a4) `(lambda (subj) (print-template
      `(descend-maintain :alt ,',a4 :agent ,subj))))
  )
(add-rule-sem '(v+args ==> on departure maintain altitude-value)
  '(lambda (a1 a2 a3 a4) `(lambda (subj) (print-template
      `(maintain :alt ,',a4 :when on-departure :agent ,subj))))
  )
(add-rule-sem '(v+args ==> maintain altitude-value at departure)
  '(lambda (a1 a2 a3 a4) `(lambda (subj) (print-template
      `(maintain :alt ,',a2 :when on-departure :agent ,subj))))
  )
(add-rule-sem '(v+args ==> maintain altitude-value)
  '(lambda (a1 a2) `(lambda (subj) (print-template
      `(maintain :alt ,',a2 :agent ,subj))))
  )
(add-rule-sem '(v+args ==> maintain heading of some-number)
  '(lambda (a1 a2 a3 a4) `(lambda (subj) (print-template
      `(maintain :heading ,',a4 :agent ,subj))))
  )
(add-rule-sem '(v+args ==> maintain this frequency)
  '(lambda (a1 a2 a3) `(lambda (subj) (print-template
      `(nop :agent ,subj)))))

;contact/monitor <speaker> [on the way in]
;remain on <freq>
;remain this frequency
(add-rule-sem '(v+args ==> contact some-speaker)
  '(lambda (a1 a2) `(lambda (subj) (print-template
      `(setfreq :goal ,',a2 :agent ,subj))))
  )
(add-rule-sem '(v+args ==> contact some-speaker on the way in)
  '(lambda (a1 a2 a3 a4 a5 a6) `(lambda (subj) (print-template
      `(setfreq :goal ,',a2 :when on-way-in :agent ,subj))))
  )
(add-rule-sem '(v+args ==> monitor some-speaker)
  '(lambda (a1 a2) `(lambda (subj) (print-template
      `(setfreq :goal ,',a2 :agent ,subj))))
  )
(add-rule-sem '(v+args ==> monitor some-speaker on the way in)
  '(lambda (a1 a2 a3 a4 a5 a6) `(lambda (subj) (print-template
      `(setfreq :goal ,',a2 :when on-way-in :agent ,subj))))
  )
(add-rule-sem '(v+args ==> remain on frequency-value)
  '(lambda (a1 a2 a3) `(lambda (subj) (print-template
      `(setfreq :goal ,',a3 :agent ,subj))))
  )
(add-rule-sem '(v+args ==> remain this frequency)
  '(lambda (a1 a2 a3) `(lambda (subj) (print-template
      `(nop :agent ,subj)))))

;clear <rw/tw>
;exit on <tw/rw>
;exit the ramp
;exit the ramp behind <ac>
(add-rule-sem '(v+args ==> clear twrw)
  '(lambda (a1 a2) `(lambda (subj) (print-template
      `(leave :goal ,',a2 :agent ,subj))))
  )
(add-rule-sem '(v+args ==> exit on twrw)
  '(lambda (a1 a2 a3) `(lambda (subj) (print-template
      `(go :on ,',a3 :agent ,subj))))
  )
(add-rule-sem '(v+args ==> exit the ramp)
  '(lambda (a1 a2 a3) `(lambda (subj) (print-template
      `(leave :goal ramp :agent ,subj))))
  )
(add-rule-sem '(v+args ==> exit the ramp behind some-plane)
  '(lambda (a1 a2 a3 a4 a5) `(lambda (subj)
    (print-template `(leave  :goal ramp  :agent ,subj))
    (print-template `(behind :goal ,',a5 :agent ,subj))
   )))

;fall in behind <ac>
;get behind <ac>
;give way to <ac>
;after <ac>
;follow <ac>
;follow in behind <ac>
;let <ac> turn in front of you
;hold for <ac>
;<ac> is going in front of you
(add-rule-sem '(v+args ==> fall in behind some-plane)
  '(lambda (a1 a2 a3 a4) `(lambda (subj) (print-template
      `(behind :goal ,',a4 :agent ,subj))))
  )
(add-rule-sem '(v+args ==> get behind some-plane)
  '(lambda (a1 a2 a3) `(lambda (subj) (print-template
      `(behind :goal ,',a3 :agent ,subj))))
  )
(add-rule-sem '(v+args ==> give way to some-plane)
  '(lambda (a1 a2 a3 a4) `(lambda (subj) (print-template
      `(behind :goal ,',a4 :agent ,subj))))
  )
(add-rule-sem '(v+args ==> after some-plane)
  '(lambda (a1 a2) `(lambda (subj) (print-template
      `(behind :goal ,',a2 :agent ,subj))))
  )
(add-rule-sem '(v+args ==> follow some-plane)
  '(lambda (a1 a2) `(lambda (subj) (print-template
      `(behind :goal ,',a2 :agent ,subj))))
  )
(add-rule-sem '(v+args ==> follow in behind some-plane)
  '(lambda (a1 a2 a3 a4) `(lambda (subj) (print-template
      `(behind :goal ,',a4 :agent ,subj))))
  )
(add-rule-sem '(v+args ==> let some-plane turn in front of you)
  '(lambda (a1 a2 a3 a4 a5 a6 a7) `(lambda (subj) (print-template
      `(behind :goal ,',a2 :agent ,subj))))
  )
(add-rule-sem '(v+args ==> hold for some-plane)
  '(lambda (a1 a2 a3) `(lambda (subj) (print-template
      `(behind :goal ,',a3 :agent ,subj))))
  )
(add-rule-sem '(v+args ==> some-plane is going in front of you)
  '(lambda (a1 a2 a3 a4 a5 a6 a7) `(lambda (subj) (print-template
      `(behind :goal ,',a1 :agent ,subj))))
  )
(add-rule-sem '(v+args ==> follow some-plane ahead to the runway)
  '(lambda (a1 a2 a3 a4 a5 a6) `(lambda (subj)
     (print-template  `(behind :goal ,',a2 :agent ,subj))
     (print-template  `(go :on (runway)    :agent ,subj))
  )))

;hold
;hold short
;hold short of <tw/rw>
;hold short of <tw/rw> on <tw/rw>
;hold short of <tw/rw> for spacing
;hold short of <tw/rw> for the <ac>
;hold <num>
;hold heading <num>
(add-rule-sem '(v+args ==> hold)
  '(lambda (a1) `(lambda (subj) (print-template
      `(hold :agent ,subj))))
  )
(add-rule-sem '(v+args ==> hold short)
  '(lambda (a1 a2) `(lambda (subj) (print-template
      `(hold-short :agent ,subj))))
  )
(add-rule-sem '(v+args ==> hold short of twrw)
  '(lambda (a1 a2 a3 a4) `(lambda (subj) (print-template
      `(hold-short :of ,',a4 :agent ,subj))))
  )
(add-rule-sem '(v+args ==> hold short of twrw on twrw)
  '(lambda (a1 a2 a3 a4 a5 a6) `(lambda (subj) (print-template
      `(hold-short :of ,',a4 :on ,',a6 :agent ,subj))))
  )
(add-rule-sem '(v+args ==> hold short of twrw for spacing)
  '(lambda (a1 a2 a3 a4 a5 a6) `(lambda (subj) (print-template
      `(hold-short :of ,',a4 :agent ,subj))))
  )
(add-rule-sem '(v+args ==> hold short of twrw for some-plane)
  '(lambda (a1 a2 a3 a4 a5 a6) `(lambda (subj) (print-template
      `(hold-short :of ,',a4 :for ,',a6 :agent ,subj))))
  )
(add-rule-sem '(v+args ==> hold some-number)
  '(lambda (a1 a2) `(lambda (subj) (print-template
      `(hold-heading :heading (heading :to ,',a2) :agent ,subj))))
  )
(add-rule-sem '(v+args ==> hold heading some-number)
  '(lambda (a1 a2 a3) `(lambda (subj) (print-template
      `(hold-heading :heading (heading :to ,',a3) :agent ,subj)))))

;position
;taxi into/to position
(add-rule-sem '(v+args ==> position)
  '(lambda (a1) `(lambda (subj) (print-template
      `(position :agent ,subj))))
  )
(add-rule-sem '(v+args ==> taxi into position)
  '(lambda (a1 a2 a3) `(lambda (subj) (print-template
      `(position :agent ,subj))))
  )
(add-rule-sem '(v+args ==> taxi to position)
  '(lambda (a1 a2 a3) `(lambda (subj) (print-template
      `(position :agent ,subj)))))

;you are <priority>
(add-rule-sem '(v+args ==> be-verb priority)
  '(lambda (a1 a2) `(lambda (subj) (print-template
      `(set-priority :new ,',a2 :agent ,subj)))))

;land
;land on? <rw>
;land on? <rw> left/right
(add-rule-sem '(v+args ==> land)
  '(lambda (a1) `(lambda (subj) (print-template
      `(land :agent ,subj))))
  )
(add-rule-sem '(v+args ==> land on? rw)
  '(lambda (a1 a2 a3) `(lambda (subj) (print-template
      `(land :on ,',a3 :agent ,subj))))
  )
(add-rule-sem '(v+args ==> land on? rw left-or-right)
  '(lambda (a1 a2 a3 a4) `(lambda (subj) (print-template
      `(land ,',a4 ,',a3 :agent ,subj)))))

;takeoff on? <rw>
(add-rule-sem '(v+args ==> takeoff on? rw)
  '(lambda (a1 a2 a3) `(lambda (subj) (print-template
      `(takeoff :on ,',a3 :agent ,subj)))))

;taxi ahead
;move as soon as you can
;move ahead
;move ahead before <ac>
;right in front of <ac>
;go [[straight] ahead] until you make it in front of <ac>
(add-rule-sem '(v+args ==> taxi ahead)
  '(lambda (a1 a2) `(lambda (subj) (print-template
      `(move :agent ,subj))))
  )
(add-rule-sem '(v+args ==> move as soon as you can)
  '(lambda (a1 a2 a3 a4 a5 a6) `(lambda (subj) (print-template
      `(move :agent ,subj))))
  )
(add-rule-sem '(v+args ==> move ahead)
  '(lambda (a1 a2) `(lambda (subj) (print-template
      `(move :agent ,subj))))
  )
(add-rule-sem '(v+args ==> move ahead before some-plane)
  '(lambda (a1 a2 a3 a4) `(lambda (subj)
    (print-template `(move              :agent ,subj))
    (print-template `(ahead :goal ,',a4 :agent ,subj))
  )))
(add-rule-sem '(v+args ==> right in front of some-plane)
  '(lambda (a1 a2 a3 a4 a5) `(lambda (subj)
    (print-template `(move              :agent ,subj))
    (print-template `(ahead :goal ,',a5 :agent ,subj))
  )))
(add-rule-sem '(v+args ==> go until you make it in front of some-plane)
  '(lambda (a1 a2 a3 a4 a5 a6 a7 a8 a9) `(lambda (subj)
    (print-template `(move              :agent ,subj))
    (print-template `(ahead :goal ,',a9 :agent ,subj))
  )))
(add-rule-sem
  '(v+args ==> go straight? ahead until you make it in front of some-plane)
  '(lambda (a1 a2 a3 a4 a5 a6 a7 a8 a9 a10 a11) `(lambda (subj)
    (print-template `(move               :agent ,subj))
    (print-template `(ahead :goal ,',a11 :agent ,subj))
  )))

;cross <num> at <num> knot(s)
;cross <num> at <num> foot/feet
;cross <tw/rw>
;cross <tw/rw> at <tw/rw>
(add-rule-sem '(v+args ==> cross some-number at some-number knot)
  '(lambda (a1 a2 a3 a4 a5) `(lambda (subj) (print-template
      `(cross :fix ,',a2 :at-speed ,',a4 :agent ,subj))))
  )
(add-rule-sem '(v+args ==> cross some-number at some-number foot)
  '(lambda (a1 a2 a3 a4 a5) `(lambda (subj) (print-template
      `(cross :fix ,',a2 :at-altitude ,',a4 :agent ,subj))))
  )
(add-rule-sem '(v+args ==> cross twrw)
  '(lambda (a1 a2) `(lambda (subj) (print-template
      `(cross :road ,',a2 :agent ,subj))))
  )
(add-rule-sem '(v+args ==> cross twrw at twrw)
  '(lambda (a1 a2 a3 a4) `(lambda (subj) (print-template
      `(cross :road ,',a2 :at ,',a4 :agent ,subj)))))

;encounter/see <ac>
;expect traffic on <tw/rw>
;expect <acname> on <tw/rw>
;expect <num> feet <num> minute(s) after departure
(add-rule-sem '(v+args ==> encounter some-plane)
  '(lambda (a1 a2) `(lambda (subj) (print-template
      `(expect :plane ,',a2 :agent ,subj))))
  )
(add-rule-sem '(v+args ==> see some-plane)
  '(lambda (a1 a2) `(lambda (subj) (print-template
      `(expect :plane ,',a2 :agent ,subj))))
  )
(add-rule-sem '(v+args ==> expect traffic on twrw)
  '(lambda (a1 a2 a3 a4) `(lambda (subj) (print-template
      `(expect :on ,',a4 :agent ,subj))))
  )
(add-rule-sem '(v+args ==> expect some-plane on twrw)
  '(lambda (a1 a2 a3 a4) `(lambda (subj) (print-template
      `(expect :plane ,',a2 :on ,',a4 :agent ,subj))))
  )
(add-rule-sem
  '(v+args ==> expect some-number foot some-number minute after departure)
  '(lambda (a1 a2 a3 a4 a5 a6 a7) `(lambda (subj) (print-template
      `(lock-altitude :alt ,',a2 :time ,',a4 :agent ,subj)))))

;clear for takeoff
(add-rule-sem '(v+args ==> clear for takeoff)
  '(lambda (a1 a2 a3) `(lambda (subj) (print-template
      `(enable-takeoff :agent ,subj)))))

;go/taxi [straight] down <tw/rw>
;go/taxi [ahead] on/to <tw/rw>
;go/taxi to <tw/rw> via <tw/rw>
;go/taxi via <tw/rw>
(add-rule-sem '(go-taxi   ==> go  ) #'identity)
(add-rule-sem '(go-taxi   ==> taxi) #'identity)
(add-rule-sem '(v+args ==> go-taxi straight? down twrw)
  '(lambda (a1 a2 a3 a4) `(lambda (subj) (print-template
      `(go :on ,',a4 :agent ,subj))))
  )
(add-rule-sem '(v+args ==> go-taxi ahead? on-or-to twrw)
  '(lambda (a1 a2 a3 a4) `(lambda (subj) (print-template
      `(go :on ,',a4 :agent ,subj))))
  )
(add-rule-sem '(v+args ==> go-taxi to twrw via twrw)
  '(lambda (a1 a2 a3 a4 a5) `(lambda (subj) (print-template
      `(go :on ,',a3 :via ,',a5 :agent ,subj))))
  )
(add-rule-sem '(v+args ==> go-taxi via twrw)
  '(lambda (a1 a2 a3) `(lambda (subj) (print-template
      `(go :via ,',a3 :agent ,subj)))))

;clear [direct] to <rw/tw>
;go/taxi/then <tw/rw>
;continue on <tw/rw>
;continue on <tw/rw> until <tw/rw>
(add-rule-sem '(go-taxi-then ==> go  ) #'identity)
(add-rule-sem '(go-taxi-then ==> taxi) #'identity)
(add-rule-sem '(go-taxi-then ==> then) #'identity)
(add-rule-sem '(v+args ==> clear to twrw)
  '(lambda (a1 a2 a3) `(lambda (subj) (print-template
      `(go :on ,',a3 :agent ,subj))))
  )
(add-rule-sem '(v+args ==> clear direct to twrw)
  '(lambda (a1 a2 a3 a4) `(lambda (subj) (print-template
      `(go :on ,',a4 :agent ,subj))))
  )
(add-rule-sem '(v+args ==> go-taxi-then twrw)
  '(lambda (a1 a2) `(lambda (subj) (print-template
      `(go :on ,',a2 :agent ,subj))))
  )
(add-rule-sem '(v+args ==> continue on twrw)
  '(lambda (a1 a2 a3) `(lambda (subj) (print-template
      `(go :on ,',a3 :agent ,subj))))
  )
(add-rule-sem '(v+args ==> continue on twrw until twrw)
  '(lambda (a1 a2 a3 a4 a5) `(lambda (subj) (print-template
      `(go :on ,',a3 :until ,',a5 :agent ,subj)))))

;straight ahead on <tw/rw>
;straight on? to <tw/rw>
(add-rule-sem '(v+args ==> straight ahead on twrw)
  '(lambda (a1 a2 a3 a4) `(lambda (subj) (print-template
      `(go :on ,',a4 :agent ,subj))))
  )
(add-rule-sem '(v+args ==> straight on? to twrw)
  '(lambda (a1 a2 a3 a4) `(lambda (subj) (print-template
      `(go :on ,',a4 :agent ,subj)))))

;turn <heading-phrase>
(add-rule-sem '(v+args ==> turn some-heading)
  '(lambda (a1 a2) `(lambda (subj) (print-template
      `(turn :heading ,',a2 :agent ,subj)))))

;;==============================================================================

(add-rule-sem '(det? ==>    ) '(lambda ()))
(add-rule-sem '(det? ==> det)   #'identity)

(add-rule-sem '(number? ==>       ) '(lambda ()))
(add-rule-sem '(number? ==> number)   #'identity)

(add-rule-sem '(on? ==>   ) '(lambda ()))
(add-rule-sem '(on? ==> on)   #'identity)

(add-rule-sem '(straight? ==>         ) '(lambda ()))
(add-rule-sem '(straight? ==> straight)   #'identity)

(add-rule-sem '(ahead? ==>      ) '(lambda ()))
(add-rule-sem '(ahead? ==> ahead)   #'identity)

(add-rule-sem '(on-or-to ==> on) #'identity)
(add-rule-sem '(on-or-to ==> to) #'identity)

(add-rule-sem '(left-or-right ==> left ) '(lambda (x) ':left ) )
(add-rule-sem '(left-or-right ==> right) '(lambda (x) ':right) )

(add-rule-sem '(ground-or-tower ==> ground   ) '(lambda (x) 'ground   ))
(add-rule-sem '(ground-or-tower ==> tower    ) '(lambda (x) 'tower    ))
(add-rule-sem '(ground-or-tower ==> departure) '(lambda (x) 'departure))
(add-rule-sem '(ground-or-tower ==> ramp     ) '(lambda (x) 'ramp     ))

(add-rule-sem '(twrw ==> rw) #'identity)
(add-rule-sem '(twrw ==> tw) #'identity)

;==============================================================================

; INPUT:
;    <ac> = [the/a/that] <acname>
;    <ac> = [the/a/that] <acname> behind [a/the/...] <acname>
;    <ac> = [the/a/that] <acname> that is behind [a/the/...] <acname>
;    <ac> = [the/a/that] <acname> [directly] ahead of you
;    <ac> = [the/a/that] <acname> from your left/right

; OUTPUT:
;    (alaska)
;    (alaska :behind boeing)
;    (alaska :ahead  you)
;    (alaska :left   you)
;    (alaska :right  you)

(add-rule-sem '(some-plane ==> det? plane)
  '(lambda (det plane)
      `(,plane)
   ))

(add-rule-sem '(some-plane ==> det? plane behind det? plane)
  '(lambda (det1 plane1 behind det2 plane2)
      `(,plane1 :behind ,plane2)
   ))

(add-rule-sem '(some-plane ==> det? plane that is behind det? plane)
  '(lambda (det1 plane1 that is behind det2 plane2)
      `(,plane1 :behind ,plane2)
   ))

(add-rule-sem '(some-plane ==> det? plane ahead of you)
  '(lambda (det plane ahead of you)
      `(,plane :ahead you)
   ))

(add-rule-sem '(some-plane ==> det? plane directly ahead of you)
  '(lambda (det plane directly ahead of you)
      `(,plane :ahead you)
   ))

(add-rule-sem '(some-plane ==> det? plane from your left-or-right)
  '(lambda (det plane from your leftright)
      `(,plane ,leftright you)
   ))

;==============================================================================

; INPUT:
;    <priority> = <pri-either> / <pri-landing> / <pri-takeoff>
;
;    <pri-either> = number? <num>
;                 = number? <num> behind [a/the...] <acname>
;                 = number? <num> to go
;
;    <pri-landing> = number? <num> for landing
;                  = number? <num> for landing behind [a/the...] <acname>
;
;    <pri-takeoff> = number? <num> for takeoff
;                  = number? <num> for takeoff behind [a/the...] <acname>

; OUTPUT:
;    (priority 3 :event unknown/takeoff/landing)
;    (priority 3 :event unknown/takeoff/landing :behind alaska)

(add-rule-sem '(priority ==> number? some-number)
  '(lambda (a1 some-number)
      `(priority :num ,some-number :event unknown)
   ))

(add-rule-sem '(priority ==> number? some-number behind det? plane)
  '(lambda (a1 some-number behind det plane)
      `(priority :num ,some-number :event unknown :behind ,plane)
   ))

(add-rule-sem '(priority ==> number? some-number to go)
  '(lambda (a1 some-number to go)
      `(priority :num ,some-number :event unknown)
   ))

(add-rule-sem '(priority ==> number? some-number for landing)
  '(lambda (a1 some-number for landing)
      `(priority :num ,some-number :event landing)
   ))

(add-rule-sem '(priority ==> number? some-number for landing behind det? plane)
  '(lambda (a1 some-number for landing behind det plane)
      `(priority :num ,some-number :event landing :behind ,plane)
   ))

(add-rule-sem '(priority ==> number? some-number for takeoff)
  '(lambda (a1 some-number for takeoff)
      `(priority :num ,some-number :event takeoff)
   ))

(add-rule-sem '(priority ==> number? some-number for takeoff behind det? plane)
  '(lambda (a1 some-number for takeoff behind det plane)
      `(priority :num ,some-number :event takeoff :behind ,plane)
   ))

;==============================================================================

; INPUT:
;   <heading-phrase> = heading of <num>
;   <heading-phrase> = heading <num>
;   <heading-phrase> = to heading <num>
;   <heading-phrase> = <num>
;   <heading-phrase> = <left/right> <num> degree(s)
;   <heading-phrase> = <left/right> <num> degree(s) to heading <num>
;   <heading-phrase> = <left/right> to heading <num>

; OUTPUT:
;    (heading :to    3)
;    (heading :left  3)
;    (heading :right 3)
;    (heading :left  3        :to 10)
;    (heading :right 3        :to 10)
;    (heading :left  unknown  :to 10)
;    (heading :right unknown  :to 10)

(add-rule-sem '(some-heading ==> heading of some-number)
  '(lambda (heading of some-number)
      `(heading :to ,some-number)
   ))

(add-rule-sem '(some-heading ==> heading some-number)
  '(lambda (heading some-number)
      `(heading :to ,some-number)
   ))

(add-rule-sem '(some-heading ==> to heading some-number)
  '(lambda (to heading some-number)
      `(heading :to ,some-number)
   ))

(add-rule-sem '(some-heading ==> some-number)
  '(lambda (some-number)
      `(heading :to ,some-number)
   ))

(add-rule-sem '(some-heading ==> left-or-right some-number degree)
  '(lambda (leftright some-number degree)
      `(heading ,leftright ,some-number)
   ))

(add-rule-sem '(some-heading ==> left-or-right some-number degree to heading some-number)
  '(lambda (leftright number1 degree to heading number2)
      `(heading ,leftright ,number1 :to ,number2)
   ))

(add-rule-sem '(some-heading ==> left-or-right to heading some-number)
  '(lambda (leftright to heading number)
      `(heading ,leftright unknown :to ,number)
   ))

;==============================================================================

; INPUT:
;   <speed value> = <num>
;   <speed value> = <num> knot(s)

; OUTPUT:
;   (<num>)

(add-rule-sem '(speed-value ==> some-number)       '(lambda (n)   `( ,n)))
(add-rule-sem '(speed-value ==> some-number knot)  '(lambda (n k) `( ,n)))

;==============================================================================

; INPUT:
;   <altitude value> = <num>
;   <altitude value> = <num> foot/feet

; OUTPUT:
;   (<num>)

(add-rule-sem '(altitude-value ==> some-number)      '(lambda (n)   `( ,n)))
(add-rule-sem '(altitude-value ==> some-number foot) '(lambda (n f) `( ,n)))

;==============================================================================

; INPUT:
;   <rw> = runway <number>
;   <rw> = <number>
;   <rw> = the runway
; OUTPUT:
;   (runway 3)
;   (runway)

(add-rule-sem '(rw ==> runway some-number) '(lambda (a b) `(runway :num ,b)))
(add-rule-sem '(rw ==> some-number)        '(lambda (a)   `(runway :num ,a)))
(add-rule-sem '(rw ==> the runway)         '(lambda (a b) `(runway        )))

;==============================================================================

; INPUT:
;   <tw> = taxiway <alpha>
;   <tw> = <alpha>
;   <tw> = the taxiway
; OUTPUT:
;   (taxiway alpha)
;   (taxiway)

(add-rule-sem '(tw ==> taxiway letter) '(lambda (a1 a2) `(taxiway :num ,a2)))
(add-rule-sem '(tw ==> letter)         '(lambda (a1)    `(taxiway :num ,a1)))
(add-rule-sem '(tw ==> the taxiway)    '(lambda (a1 a2) `(taxiway         )))

;==============================================================================

; INPUT:
;   <altimeter value> = <num>

; OUTPUT:
;   (<num>)

(add-rule-sem '(altimeter-value ==> some-number)     '(lambda (n) `( ,n)))

;==============================================================================

; INPUT:
;   <frequency> = <num>

; OUTPUT:
;   (<num>)

(add-rule-sem '(frequency-value ==> some-number)
   '(lambda (n) `(speaker :freq ,n)))

;==============================================================================

; INPUT:
;   <speaker> = departure/ground/tower/ramp
;   <speaker> = departure/ground/tower/ramp at/on <freq>
;   <speaker> = <freq>
;   <speaker> = this frequency

; OUTPUT:
;   (speaker :name departure/ground/tower/ramp)
;   (speaker :name departure/ground/tower/ramp :freq 4)
;   (speaker :freq 4)
;   (speaker :freq nochange)

(add-rule-sem '(some-speaker ==> ground-or-tower)
   '(lambda (gt) `(speaker :name ,gt)))

(add-rule-sem '(some-speaker ==> ground-or-tower at some-number)
   '(lambda (gt at num) `(speaker :name ,gt :freq ,num)))

(add-rule-sem '(some-speaker ==> ground-or-tower on some-number)
   '(lambda (gt on num) `(speaker :name ,gt :freq ,num)))

(add-rule-sem '(some-speaker ==> some-number)
   '(lambda (num) `(speaker :freq ,num)))

(add-rule-sem '(some-speaker ==> this frequency)
   '(lambda (this freq) `(speaker :freq nochange)))

;==============================================================================

; INPUT:
;   <time-value> = at <num> o'clock
;   <time-value> = in <num> minute(s)

; OUTPUT:
;   (timevalue :at $3)
;   (timevalue :in $3)

(add-rule-sem '(time-value ==> at some-number o^clock)
   '(lambda (at num oclock) `(timevalue :at ,num)))

(add-rule-sem '(time-value ==> in some-number minute)
   '(lambda (in num units) `(timevalue :in ,num)))

;==============================================================================
