
#ifndef include_plane_h
#define include_plane_h

//=============================================================================

typedef enum
 {
  NO_CHANGE,
  LAST_CHANGE_IS_FREQ,
  LAST_CHANGE_IS_SPEED,
  LAST_CHANGE_IS_DEST
 }
plane_change_type;

typedef enum
 {
  PARK,      // Still in the parking lot
  TAXI,      // Going towards the runup area
  RUNUP,     // Performing runup
  HOLDSHORT, // Run up is done. Now it's holding short
  POSITION,  // It's at the runway position
  TAKINGOFF, // It's taking off...
  OFFSCREEN, // It's in air (and thus "off screen")
  LANDING    // It's landing...
 }
plane_state;

//=============================================================================

class plane
 {
  public:

    int speed;  // Current speed. 0 if I'm stopped.

  private:

    plane_state state;
    int statetime; // How many UNITS of time has the plane stayed in this state

    int id;              // My aircraft number (1,2,3...)
    int altimeter;       // My altimeter settings
    plane_change_type newly_changed;

    int speed_locked;  // Whether or not the speed is locked
    int targetSpeed;   // My current target speed
    int oldtargetSpeed;

    int freq;     // My current frequency
    int oldfreq;

    int altitude_locked; // Whether or not the altitude is locked
    int target_altitude; // My current target altitude

    int x,y,deg;// My current location+angle (in absolute coordinates)
    int src;    // My current location (as one of the vertex on the airport)
    int dest;   // My destination (as one of the vertex on the airport)
    int* path;  // My current path

  private:

    void set_path(int*); // Private helper function. See code for detail.

  public:

    // Constructs an airplane, located at "start"
    plane(int start);

    // Return a pointer to the i-th plane. (i=1..)
    // If there's no such plane, it returns NULL.
    static plane* plane::getplane(int i);

    // Get/Set the "newly changed" flag
    plane_change_type get_newly_changed(void)   { return newly_changed; }
    void set_newly_changed(plane_change_type x) { newly_changed=x; }

    // Read/Set the state, and read/increment the State Time
    plane_state get_state(void)   { return state; }
    void set_state(plane_state x) { if (state!=x) { state=x; statetime=0; } }
    int get_time(void)            { return statetime; }

    // Set the current Altimeter
    void set_altimeter(int x) { altimeter=x; }

    // Get/Set the current destination
    void set_dest(int x);
    void via(int taxiway); // Re-plot a path that goes via "taxiway".

    // Get/Set the current Target altitude
    int  get_target_altitude(void)  { return target_altitude; }
    void set_target_altitude(int x) { target_altitude=x; }
    int  is_altitude_locked(void)   { return altitude_locked; }
    void lock_altitude(void)        { altitude_locked=1; }
    void unlock_altitude(void)      { altitude_locked=0; }

    // Get/Set the current target speed, or swap the old/new target speeds
    int  get_target_speed(void) { return targetSpeed; }
    void set_target_speed(int x)
     {
      newly_changed=LAST_CHANGE_IS_SPEED;
      oldtargetSpeed=targetSpeed; targetSpeed=x;
     }
    void swap_speed(void)
     {
      int x=oldtargetSpeed; oldtargetSpeed=targetSpeed; targetSpeed=x;
     }
    int  is_speed_locked(void)   { return speed_locked; }
    void lock_speed(void)        { speed_locked=1; }
    void unlock_speed(void)      { speed_locked=0; }

    // Get/Set the current frequency, or swap the old/new frequencies
    int  get_freq(void)  { return freq; }
    void set_freq(int x)
     {
      newly_changed=LAST_CHANGE_IS_FREQ; oldfreq=freq; freq=x;
     }
    void swap_freq(void) { int x=oldfreq; oldfreq=freq; freq=x; }

    // Let the "otherplane" have lower priority than me
    void ahead (int otherplane) { }

    // Let the "otherplane" have higher priority than me
    void behind(int otherplane) { }

    // Undraw/Draw this airplane.
    void hide(void);
    void show(void);

    // Advance the simulation for this airplane.
    int advance(void);

    // Commence the landing process for this plane.
    void land(int runway);
 };

//=============================================================================

#endif
