
#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include <stdlib.h>

#include "reparse.h"

//=============================================================================

// Returns TRUE if "xx" is a letter of the alphabet
#define local_is_alpha(xx) (((xx)>='A' && (xx)<='Z')||((xx)>='a' && (xx)<='z'))

// Returns TRUE if "xx" is a numeric digit
#define local_is_digit(xx) ((xx)>='0' && (xx)<='9')

// Returns TRUE if "xx" is a "linebreak", "space", "tab",... etc.
#define local_is_space(xx) ((xx)>0 && (xx)<=32)

//=============================================================================

// This functions takes a string "(abc :field1 value1 :field2 value2)"
// and returns a "reparse" structure that
// has "abc" as its head, and has 2 subentries.
// The 2 subentries are "field1,value1" and "field2,value2".
//
// Note: If a subentry has parenthesis,
// then the subexpression will be parsed into a "reparse" structure
// as well.
//
// Thus, this function recursively calls itself.
//
// When its done, it will store the "pointer" to the end of string
// into (*retptr).
//
static reparse* local_parse(char* ptr,char** retptr)
 {
  reparse* answer; char* head; int n=0;
  struct reparse_trio* array=0; int done=0;
  // Skip leading spaces...
  while(local_is_space(*ptr)) ptr=ptr+1;
  // Get the leading '('
  if (*ptr!='(') return 0; ptr=ptr+1;
  while(local_is_space(*ptr)) ptr=ptr+1;
  // Get the "head"
  if (!local_is_alpha(*ptr) && !local_is_digit(*ptr)) return 0;
  head=ptr;
  while(!local_is_space(*ptr) && *ptr && *ptr!=')') ptr=ptr+1;
  if (*ptr==0) return 0;
  if (*ptr==')') { *ptr=0; ptr=ptr+1; goto good; }
  *ptr=0; ptr=ptr+1; while(local_is_space(*ptr)) ptr=ptr+1;
  // Get the arguments...
  while(!done)
   {
    char* a; char* b; reparse* c; while(local_is_space(*ptr)) ptr=ptr+1;
    if (*ptr==')') { *ptr=0; ptr=ptr+1; break; }
    if (*ptr!=':') goto bad;
    ptr=ptr+1; a=ptr;
    while(*ptr && !local_is_space(*ptr) && *ptr!=')') ptr=ptr+1;
    if (*ptr==0 || *ptr==')') goto bad;
    *ptr=0; ptr=ptr+1; while(local_is_space(*ptr)) ptr=ptr+1;
    if (*ptr=='(')
     {
      char* ret; c=local_parse(ptr,&ret); if (c==0) goto bad;
      b=0; ptr=ret;
     }
    else if (local_is_alpha(*ptr) || local_is_digit(*ptr))
     {
      c=0; b=ptr; while(*ptr && !local_is_space(*ptr) && *ptr!=')') ptr=ptr+1;
      if (*ptr==0) goto bad;
      if (*ptr==')') { *ptr=0; ptr=ptr+1; done=1; } else { *ptr=0; ptr=ptr+1; }
     }
    else return 0;
    if (n<=0) array=(struct reparse_trio*) malloc(sizeof(struct reparse_trio));
    else
     {
      array=(struct reparse_trio*)
         realloc(array,(n+1)*sizeof(struct reparse_trio));
     }
    if (array==0) exit(1);
    array[n].key=a; array[n].value=b; array[n].subvalue=c; n=n+1;
   }
  good: answer=(reparse*)malloc(sizeof(reparse)); if (answer==0) exit(1);
  answer->head=head; answer->n=n; answer->sub=array; *retptr=ptr;
  return answer;
  bad: return 0;
 }

//=============================================================================

reparse* reparse_parse(char* ptr)
 {
  char* unused=NULL; return local_parse(ptr,&unused);
 }

//=============================================================================
