
#ifndef include_reparse_h
#define include_reparse_h

//=============================================================================

struct reparse_trio;

typedef struct
 {
  char* head;
  int n;
  struct reparse_trio* sub;
 }
reparse;

struct reparse_trio
 {
  char* key; char* value; reparse* subvalue;
 };


// Each response is parsed into a "reparse" structure.
//
// For example, say... (monitor :speaker ground :frequency 10)
// It will be a reparse structure
// with its head   == "monitor"
// with its  n     == 2
// with its sub[0] == ("speaker"   , "ground" , NULL)
// with its sub[1] == ("frequency" , "10"     , NULL)


// If a value contains parenthesis, then it will
// be represented as a sub-structure as well.
//
// For example, say... (land :on (runway :num 2) :behind Boeing)
// It will be a reparse structure
// with its head   == "land"
// with its  n     == 2
// with its sub[0] == ("on"     , NULL     , X)
// with its sub[1] == ("behind" , "Boeing" , NULL)
// ..
// Where "X" is a sub-reparse structure
// with its head   == "runway"
// with its  n     == 1
// with its sub[0] == ("num" , "2" , NULL)


//=============================================================================

// This is the interface function to the reparser.
// It takes the output of the parser, eg. "(land :on (runway :num 2))"
// and it produces a "reparse" structure containing all fields and values.
//
// If the input string does not conform to our format,
// (for example, if the input string has un-balanced parenthesis),
// then we'll return NULL.
//
reparse* reparse_parse(char* ptr);

//=============================================================================

#endif
